@@ -1,6 +1,6 @@
 /* xfce4-mpc-plugin.c
  *
- * Copyright (c) 2006-2007 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
+ * Copyright (c) 2006-2008 Landry Breuil (landry at fr.homeunix.org / gaston at gcu.info)
  * This code is licenced under a BSD-style licence.
  * (OpenBSD variant modeled after the ISC licence)
  * All rights reserved.
@@ -65,7 +65,13 @@ mpc_set_size (XfcePanelPlugin * plugin, int size, t_mpc * mpc)
          gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), size-4 , -1);
    }
    else
+   {
       gtk_container_set_border_width (GTK_CONTAINER (mpc->frame), 0);
+      if (xfce_panel_plugin_get_orientation (plugin) == GTK_ORIENTATION_HORIZONTAL)
+         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), -1, size);
+      else
+         gtk_widget_set_size_request (GTK_WIDGET (mpc->frame), size, -1);
+   }
    return TRUE;
 }
 
@@ -300,6 +306,20 @@ mpc_repeat_toggled(GtkWidget *widget, t_mpc* mpc)
    mpd_player_set_repeat(mpc->mo, gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget)));
 }
 
+void
+format_song_display(mpd_Song* song, gchar* str)
+{
+   /* buf may contain stuff, care to append text */
+   if (!song->artist || !song->title)
+      g_sprintf(str,"%s%s", str, song->file);
+   else if (!song->album)
+      g_sprintf(str,"%s%s - %s", str, song->artist, song->title);
+   else if (!song->track)
+      g_sprintf(str,"%s%s - %s -/- %s", str, song->artist, song->album, song->title);
+   else
+      g_sprintf(str,"%s%s - %s -/- (#%s) %s", str, song->artist, song->album, song->track, song->title);
+}
+
 static void
 enter_cb(GtkWidget *widget, GdkEventCrossing* event, t_mpc* mpc)
 {
@@ -322,23 +342,23 @@ enter_cb(GtkWidget *widget, GdkEventCrossing* event, t_mpc* mpc)
    switch (mpd_player_get_state(mpc->mo))
    {
       case MPD_PLAYER_PLAY:
-         g_sprintf(str, "%s - Mpd Playing",str);
+         g_sprintf(str, "%s - Mpd Playing\n",str);
          break;
       case MPD_PLAYER_PAUSE:
-         g_sprintf(str, "%s - Mpd Paused",str);
+         g_sprintf(str, "%s - Mpd Paused\n",str);
          break;
       case MPD_PLAYER_STOP:
-         g_sprintf(str, "%s - Mpd Stopped",str);
+         g_sprintf(str, "%s - Mpd Stopped\n",str);
          break;
       default:
-         g_sprintf(str, "%s - Mpd state ?",str);
+         g_sprintf(str, "%s - Mpd state ?\n",str);
          break;
    }
    song = mpd_playlist_get_current_song(mpc->mo);
-   if (song)
-      g_sprintf(str,"%s - [%s - %s] -/- (#%s) %s", str, song->artist, song->album, song->track, song->title);
+   if (song && song->id != -1)
+      format_song_display(song, str);
    else
-      g_sprintf(str,"%s - Failed to get song info ?", str);
+      g_sprintf(str,"%sFailed to get song info ?", str);
 
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->random), mpd_player_get_random(mpc->mo));
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(mpc->repeat), mpd_player_get_repeat(mpc->mo));
@@ -367,7 +387,7 @@ static void
 show_playlist (t_mpc* mpc)
 {
    DBG("!");
-   GtkWidget *scrolledwin,*treeview;
+   GtkWidget *scrolledwin, *treeview;
    GtkListStore *liststore;
    GtkTreeIter iter;
    GtkTreePath *path_to_cur;
@@ -376,55 +396,66 @@ show_playlist (t_mpc* mpc)
    int current;
    MpdData *mpd_data;
 
-   mpc->playlist = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-   gtk_window_set_default_size(GTK_WINDOW(mpc->playlist), 300, 530);
-   gtk_window_set_icon_name(GTK_WINDOW(mpc->playlist),"xfce-multimedia");
-   gtk_window_set_title(GTK_WINDOW(mpc->playlist),_("Mpd playlist"));
-   gtk_window_set_keep_above(GTK_WINDOW(mpc->playlist),TRUE); /* UGLY !!! */
-   scrolledwin = gtk_scrolled_window_new(NULL, NULL);
-   gtk_container_add(GTK_CONTAINER(mpc->playlist),GTK_WIDGET(scrolledwin));
-
-   treeview = gtk_tree_view_new ();
-   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
-   gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
-   g_signal_connect(treeview, "row-activated", G_CALLBACK(playlist_title_dblclicked), mpc);
-   gtk_container_add(GTK_CONTAINER(scrolledwin),treeview);
-
-   liststore = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
-   gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL(liststore));
-
-   renderer = gtk_cell_renderer_pixbuf_new ();
-   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, "Icon", renderer, "stock-id", 0, NULL);
-   renderer = gtk_cell_renderer_text_new ();
-   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, "Title", renderer, "text", 1, NULL);
-
-   if (!mpc_plugin_reconnect(mpc))
+   if (mpc->playlist)
    {
-      gtk_widget_destroy(mpc->playlist);
+      gtk_window_present(GTK_WINDOW(mpc->playlist));
       return;
    }
-
-   current = mpd_player_get_current_song_pos (mpc->mo);
-   DBG ("Current song pos in the list: %d", current);
-   mpd_data = mpd_playlist_get_changes (mpc->mo, -1);
-   DBG ("Got playlist, creating window");
-   do
+   /* create playlist window only if playlist is not empty */
+   if (0 != mpd_playlist_get_playlist_length(mpc->mo))
    {
-      g_sprintf(str,"%s - %s", mpd_data->song->artist, mpd_data->song->title);
+      DBG ("Creating playlist window");
+      mpc->playlist = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+      gtk_window_set_default_size(GTK_WINDOW(mpc->playlist), 400, 600);
+      gtk_window_set_icon_name(GTK_WINDOW(mpc->playlist),"xfce-multimedia");
+      gtk_window_set_title(GTK_WINDOW(mpc->playlist),_("Mpd playlist"));
+      g_signal_connect(mpc->playlist, "destroy", G_CALLBACK(gtk_widget_destroyed), &mpc->playlist);
+      scrolledwin = gtk_scrolled_window_new(NULL, NULL);
+      gtk_container_add(GTK_CONTAINER(mpc->playlist),GTK_WIDGET(scrolledwin));
 
-      gtk_list_store_append (liststore, &iter);
-      if (current == mpd_data->song->pos)
+      treeview = gtk_tree_view_new ();
+      gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
+      gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
+      g_signal_connect(treeview, "row-activated", G_CALLBACK(playlist_title_dblclicked), mpc);
+      gtk_container_add(GTK_CONTAINER(scrolledwin),treeview);
+
+      liststore = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT);
+      gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL(liststore));
+
+      renderer = gtk_cell_renderer_pixbuf_new ();
+      gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, "Icon", renderer, "stock-id", 0, NULL);
+      renderer = gtk_cell_renderer_text_new ();
+      gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1, "Title", renderer, "text", 1, NULL);
+
+      if (!mpc_plugin_reconnect(mpc))
       {
-         gtk_list_store_set (liststore, &iter, 0, "gtk-media-play", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
-         path_to_cur = gtk_tree_model_get_path(GTK_TREE_MODEL(liststore), &iter);
-         gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview), path_to_cur, NULL, TRUE, 0.5, 0);
-         gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview), path_to_cur, NULL, FALSE);
-         gtk_tree_path_free(path_to_cur);
+         gtk_widget_destroy(mpc->playlist);
+         return;
       }
-      else
-         gtk_list_store_set (liststore, &iter, 0, "gtk-cdrom", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
-   } while (NULL != (mpd_data = mpd_data_get_next (mpd_data)));
-   gtk_widget_show_all(mpc->playlist);
+
+      current = mpd_player_get_current_song_pos (mpc->mo);
+      DBG ("Current song pos in the list: %d", current);
+      mpd_data = mpd_playlist_get_changes (mpc->mo, -1);
+      DBG ("Got playlist, filling treeview");
+      do
+      {
+         str[0]='\0';
+         format_song_display(mpd_data->song, str);
+
+         gtk_list_store_append (liststore, &iter);
+         if (current == mpd_data->song->pos)
+         {
+            gtk_list_store_set (liststore, &iter, 0, "gtk-media-play", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
+            path_to_cur = gtk_tree_model_get_path(GTK_TREE_MODEL(liststore), &iter);
+            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview), path_to_cur, NULL, TRUE, 0.5, 0);
+            gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview), path_to_cur, NULL, FALSE);
+            gtk_tree_path_free(path_to_cur);
+         }
+         else
+            gtk_list_store_set (liststore, &iter, 0, "gtk-cdrom", 1, str, 2, mpd_data->song->pos, 3, mpd_data->song->id, -1);
+      } while (NULL != (mpd_data = mpd_data_get_next (mpd_data)));
+      gtk_widget_show_all(mpc->playlist);
+   }
 }
 
 static void
@@ -541,7 +572,7 @@ new_button_with_img(XfcePanelPlugin * plugin, GtkWidget *parent, GtkWidget *butt
    GtkWidget *image;
 
    button = (GtkWidget*) xfce_create_panel_button();
-   image = gtk_image_new_from_stock(icon,GTK_ICON_SIZE_BUTTON);
+   image = gtk_image_new_from_stock(icon,GTK_ICON_SIZE_BUTTON); /* FIXME for small panels */
 
    gtk_button_set_image(GTK_BUTTON(button),image);
    xfce_panel_plugin_add_action_widget (plugin, button);
@@ -600,6 +631,35 @@ mpc_create (XfcePanelPlugin * plugin)
    return mpc;
 }
 
+static void
+mpc_show_about(XfcePanelPlugin *plugin, t_mpc* mpc)
+{
+   XfceAboutInfo *ainfo;
+   GdkPixbuf *icon;
+
+   if (mpc->about)
+   {
+      gtk_window_present(GTK_WINDOW(mpc->about));
+      return;
+   }
+   ainfo = xfce_about_info_new(_("Xfce4 Mpc Plugin"), PACKAGE_VERSION,
+                               _("A simple panel-plugin client for Music Player Daemon"),
+                               _("Copyright (c) 2006-2008 Landry Breuil\n"),
+                               XFCE_LICENSE_BSD);
+   xfce_about_info_add_credit(ainfo, "Landry Breuil", "landry@fr.homeunix.org", _("Maintainer, Original Author"));
+   xfce_about_info_set_homepage(ainfo, "http://goodies.xfce.org/projects/panel-plugins/xfce4-mpc-plugin");
+
+   icon = xfce_themed_icon_load("xfce-multimedia", 32);
+
+   mpc->about = xfce_about_dialog_new_with_values(NULL, ainfo, icon);
+   gtk_widget_show_all(mpc->about);
+   g_signal_connect(G_OBJECT(mpc->about), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+   g_signal_connect(G_OBJECT(mpc->about), "destroy", G_CALLBACK(gtk_widget_destroyed), &mpc->about);
+
+   if(icon)
+      g_object_unref(G_OBJECT(icon));
+}
+
 static void
 mpc_construct (XfcePanelPlugin * plugin)
 {
@@ -640,6 +700,8 @@ mpc_construct (XfcePanelPlugin * plugin)
    xfce_panel_plugin_menu_show_configure (plugin);
 
    g_signal_connect (plugin, "configure-plugin", G_CALLBACK (mpc_create_options), mpc);
+   xfce_panel_plugin_menu_show_about(plugin);
+   g_signal_connect (plugin, "about", G_CALLBACK (mpc_show_about), mpc);
 }
 
 
